@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: main.cc,v 1.8 2005/01/06 17:42:56 murrayc Exp $ */
+/* $Id: main.cc,v 1.9 2005/03/14 15:22:13 murrayc Exp $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -209,7 +209,7 @@ static gboolean glibmm_iosource_callback(GIOChannel*, GIOCondition condition, vo
   try
   {
     // Recreate the specific slot from the generic slot node.
-    return (*static_cast<sigc::slot<bool,Glib::IOCondition>*>(callback_data->node->get_slot()))
+    return (*static_cast<sigc::slot<bool, Glib::IOCondition>*>(callback_data->node->get_slot()))
                                   ((Glib::IOCondition) condition);
   }
   catch(...)
@@ -219,6 +219,26 @@ static gboolean glibmm_iosource_callback(GIOChannel*, GIOCondition condition, vo
   return 0;
 }
 
+/* Only used by SignalChildWatch::connect().
+ * These don't use Glib::Source, to avoid the unnecessary overhead
+ * of a completely unused wrapper object.
+ */
+static gboolean glibmm_child_watch_callback(GPid pid, gint child_status, void* data)
+{
+  SourceConnectionNode *const conn_data = static_cast<SourceConnectionNode*>(data);
+  
+  try {
+    //Recreate the specific slot from the generic slot node.
+    (*static_cast<sigc::slot<void, GPid, int>*>(conn_data->get_slot()))(pid, child_status);
+  }
+
+  catch(...)
+  {
+    Glib::exception_handlers_invoke();
+  }
+  return 0;
+}
+
 } // anonymous namespace
 
 
@@ -364,6 +384,40 @@ SignalIO signal_io()
   return SignalIO(0); // 0 means default context
 }
 
+/**** Glib::SignalChildWatch **************************************************/
+
+inline
+SignalChildWatch::SignalChildWatch(GMainContext* context)
+:
+  context_ (context)
+{}
+
+sigc::connection SignalChildWatch::connect(const sigc::slot<void, GPid, int>& slot,
+                                        GPid pid, int priority)
+{
+  SourceConnectionNode *const conn_node = new SourceConnectionNode(slot);
+  const sigc::connection connection(*conn_node->get_slot());
+
+  GSource *const source = g_child_watch_source_new(pid);
+ 
+  if(priority != G_PRIORITY_DEFAULT)
+    g_source_set_priority(source, priority);
+
+  g_source_set_callback(
+      source, (GSourceFunc)&glibmm_child_watch_callback, conn_node,
+      &SourceConnectionNode::destroy_notify_callback);
+
+  g_source_attach(source, context_);
+  g_source_unref(source); // GMainContext holds a reference
+
+  conn_node->install(source);
+  return connection;
+}
+
+SignalChildWatch signal_child_watch()
+{
+  return SignalChildWatch(0); // 0 means default context
+}
 
 /**** Glib::MainContext ****************************************************/
 
@@ -485,6 +539,11 @@ SignalIO MainContext::signal_io()
   return SignalIO(gobj());
 }
 
+SignalChildWatch MainContext::signal_child_watch()
+{
+  return SignalChildWatch(gobj());
+}
+
 void MainContext::reference() const
 {
   g_main_context_ref(reinterpret_cast<GMainContext*>(const_cast<MainContext*>(this)));
